#include <stdio.h>
#include <string.h>
#include <Arduino.h>
#include <../Wire/Wire.h>
#include "WM8978.h"
#include "types.h"

// WM8978 register value buffer zone (total 58 registers 0 to 57), occupies 116 bytes of memory
// Because the IIC WM8978 operation does not support read operations, so save all the register values in the local
// Write WM8978 register, synchronized to the local register values, register read, register directly back locally stored value.
// Note: WM8978 register value is 9, so use u16 storage.

static u16 WM8978_REGVAL_TBL[58]=
{
	0X0000,0X0000,0X0000,0X0000,0X0050,0X0000,0X0140,0X0000,
	0X0000,0X0000,0X0000,0X00FF,0X00FF,0X0000,0X0100,0X00FF,
	0X00FF,0X0000,0X012C,0X002C,0X002C,0X002C,0X002C,0X0000,
	0X0032,0X0000,0X0000,0X0000,0X0000,0X0000,0X0000,0X0000,
	0X0038,0X000B,0X0032,0X0000,0X0008,0X000C,0X0093,0X00E9,
	0X0000,0X0000,0X0000,0X0000,0X0003,0X0010,0X0010,0X0100,
	0X0100,0X0002,0X0001,0X0001,0X0039,0X0039,0X0039,0X0039,
	0X0001,0X0001
}; 
//WM8978 init
//ֵ:0,ʼ
//    ,
u8 WM8978_Init(void)
{
	u8 res;
	res=WM8978_Write_Reg(0,0);	//λWM8978
	if(res)return 1;			//ָʧ,WM8978쳣
	//Ϊͨ
	WM8978_Write_Reg(1,0X1B);	//R1,MICENΪ1(MICʹ),BIASENΪ1(ģ),VMIDSEL[1:0]Ϊ:11(5K)
	WM8978_Write_Reg(2,0X1B0);	//R2,ROUT1,LOUT1ʹ(Թ),BOOSTENR,BOOSTENLʹ
	WM8978_Write_Reg(3,0X6C);	//R3,LOUT2,ROUT2ʹ(ȹ),RMIX,LMIXʹ	
	WM8978_Write_Reg(6,0);		//R6,MCLKⲿṩ
	WM8978_Write_Reg(43,1<<4);	//R43,INVROUT2,
	WM8978_Write_Reg(47,1<<8);	//R47,PGABOOSTL,ͨMIC20
	WM8978_Write_Reg(48,1<<8);	//R48,PGABOOSTR,ͨMIC20
	WM8978_Write_Reg(49,1<<1);	//R49,TSDEN,ȱ 
	WM8978_Write_Reg(10,1<<3);	//R10,SOFTMUTEر,128x,SNR 
	WM8978_Write_Reg(14,1<<3);	//R14,ADC 128x
	return 0;
}
//WM8978дĴ
//reg:Ĵַ
//val:ҪдĴֵ 
//ֵ:0,ɹ;
//,
u8 WM8978_Write_Reg(u8 reg,u16 val)
{ 
	char buf[2];
	buf[0] = (reg<<1)|((val>>8)&0X01);
	buf[1] = val&0XFF;
	Wire.beginTransmission(WM8978_ADDR); //ݵ豸Ϊ4Ĵӻ
	Wire.write(buf,2);
	Wire.endTransmission();    // ֹͣ
	WM8978_REGVAL_TBL[reg]=val;	//Ĵֵ
	return 0;	
}  

// WM8978 read register
// Reads the value  of the local register buffer zone
// reg: Register Address
// Return Value: Register value
u16 WM8978_Read_Reg(u8 reg)
{  
	return WM8978_REGVAL_TBL[reg];	
} 
//WM8978 DAC/ADC
//adcen:adcʹ(1)/ر(0)
//dacen:dacʹ(1)/ر(0)
void WM8978_ADDA_Cfg(u8 dacen,u8 adcen)
{
	u16 regval;
	regval=WM8978_Read_Reg(3);	//ȡR3
	if(dacen)regval|=3<<0;		//R32λΪ1,DACR&DACL
	else regval&=~(3<<0);		//R32λ,رDACR&DACL.
	WM8978_Write_Reg(3,regval);	//R3
	regval=WM8978_Read_Reg(2);	//ȡR2
	if(adcen)regval|=3<<0;		//R22λΪ1,ADCR&ADCL
	else regval&=~(3<<0);		//R22λ,رADCR&ADCL.
	WM8978_Write_Reg(2,regval);	//R2	
}
//WM8978 ͨ 
//micen:MIC(1)/ر(0)
//lineinen:Line In(1)/ر(0)
//auxen:aux(1)/ر(0) 
void WM8978_Input_Cfg(u8 micen,u8 lineinen,u8 auxen)
{
	u16 regval;  
	regval=WM8978_Read_Reg(2);	//ȡR2
	if(micen)regval|=3<<2;		//INPPGAENR,INPPGAENL(MICPGAŴ)
	else regval&=~(3<<2);		//رINPPGAENR,INPPGAENL.
 	WM8978_Write_Reg(2,regval);	//R2 
	
	regval=WM8978_Read_Reg(44);	//ȡR44
	if(micen)regval|=3<<4|3<<0;	//LIN2INPPGA,LIP2INPGA,RIN2INPPGA,RIP2INPGA.
	else regval&=~(3<<4|3<<0);	//رLIN2INPPGA,LIP2INPGA,RIN2INPPGA,RIP2INPGA.
	WM8978_Write_Reg(44,regval);//R44
	
	if(lineinen)WM8978_LINEIN_Gain(5);//LINE IN 0dB
	else WM8978_LINEIN_Gain(0);	//رLINE IN
	if(auxen)WM8978_AUX_Gain(7);//AUX 6dB
	else WM8978_AUX_Gain(0);	//رAUX  
}
//WM8978  
//dacen:DAC()(1)/ر(0)
//bpsen:Bypass(¼,MIC,LINE IN,AUX)(1)/ر(0) 
void WM8978_Output_Cfg(u8 dacen,u8 bpsen)
{
	u16 regval=0;
	if(dacen)regval|=1<<0;	//DACʹ
	if(bpsen)
	{
		regval|=1<<1;		//BYPASSʹ
		regval|=5<<2;		//0dB
	} 
	WM8978_Write_Reg(50,regval);//R50
	WM8978_Write_Reg(51,regval);//R51 
}
//WM8978 MIC(BOOST20dB,MIC-->ADC벿ֵ)
//gain:0~63,Ӧ-12dB~35.25dB,0.75dB/Step
void WM8978_MIC_Gain(u8 gain)
{
	gain&=0X3F;
	WM8978_Write_Reg(45,gain);		//R45,ͨPGA 
	WM8978_Write_Reg(46,gain|1<<8);	//R46,ͨPGA
}
//WM8978 L2/R2(ҲLine In)(L2/R2-->ADC벿ֵ)
//gain:0~7,0ʾֹͨ,1~7,Ӧ-12dB~6dB,3dB/Step
void WM8978_LINEIN_Gain(u8 gain)
{
	u16 regval;
	gain&=0X07;
	regval=WM8978_Read_Reg(47);	//ȡR47
	regval&=~(7<<4);			//ԭ 
 	WM8978_Write_Reg(47,regval|gain<<4);//R47
	regval=WM8978_Read_Reg(48);	//ȡR48
	regval&=~(7<<4);			//ԭ 
 	WM8978_Write_Reg(48,regval|gain<<4);//R48
} 
//WM8978 AUXR,AUXL(PWMƵ)(AUXR/L-->ADC벿ֵ)
//gain:0~7,0ʾֹͨ,1~7,Ӧ-12dB~6dB,3dB/Step
void WM8978_AUX_Gain(u8 gain)
{
	u16 regval;
	gain&=0X07;
	regval=WM8978_Read_Reg(47);	//ȡR47
	regval&=~(7<<0);			//ԭ 
 	WM8978_Write_Reg(47,regval|gain<<0);//R47
	regval=WM8978_Read_Reg(48);	//ȡR48
	regval&=~(7<<0);			//ԭ 
 	WM8978_Write_Reg(48,regval|gain<<0);//R48
}  
//I2Sģʽ
//fmt:0,LSB(Ҷ);1,MSB();2,ֱ׼I2S;3,PCM/DSP;
//len:0,16λ;1,20λ;2,24λ;3,32λ;  
void WM8978_I2S_Cfg(u8 fmt,u8 len)
{
	fmt&=0X03;
	len&=0X03;//޶Χ
	WM8978_Write_Reg(4,(fmt<<3)|(len<<5));	//R4,WM8978ģʽ	
}	

//ö
//voll:(0~63)
//volr:(0~63)
void WM8978_HPvol_Set(u8 voll,u8 volr)
{
	voll&=0X3F;
	volr&=0X3F;//޶Χ
	if(voll==0)voll|=1<<6;//Ϊ0ʱ,ֱmute
	if(volr==0)volr|=1<<6;//Ϊ0ʱ,ֱmute 
	WM8978_Write_Reg(52,voll);			//R52,
	WM8978_Write_Reg(53,volr|(1<<8));	//R53,,ͬ(HPVU=1)
}
//
//voll:(0~63) 
void WM8978_SPKvol_Set(u8 volx)
{ 
	volx&=0X3F;//޶Χ
	if(volx==0)volx|=1<<6;//Ϊ0ʱ,ֱmute 
 	WM8978_Write_Reg(54,volx);			//R54,
	WM8978_Write_Reg(55,volx|(1<<8));	//R55,,ͬ(SPKVU=1)	
}
//3D
//depth:0~15(3Dǿ,0,15ǿ)
void WM8978_3D_Set(u8 depth)
{ 
	depth&=0XF;//޶Χ 
 	WM8978_Write_Reg(41,depth);	//R41,3D 	
}
//EQ/3D÷
//dir:0,ADC
//    1,DAC(Ĭ)
void WM8978_EQ_3D_Dir(u8 dir)
{
	u16 regval; 
	regval=WM8978_Read_Reg(0X12);
	if(dir)regval|=1<<8;
	else regval&=~(1<<8); 
 	WM8978_Write_Reg(18,regval);//R18,EQ1ĵ9λEQ/3D
}

//EQ1
//cfreq:ֹƵ,0~3,ֱӦ:80/105/135/175Hz
//gain:,0~24,Ӧ-12~+12dB
void WM8978_EQ1_Set(u8 cfreq,u8 gain)
{ 
	u16 regval;
	cfreq&=0X3;//޶Χ 
	if(gain>24)gain=24;
	gain=24-gain;
	regval=WM8978_Read_Reg(18);
	regval&=0X100;
	regval|=cfreq<<5;	//ýֹƵ 
	regval|=gain;		//	
 	WM8978_Write_Reg(18,regval);//R18,EQ1 	
}
//EQ2
//cfreq:Ƶ,0~3,ֱӦ:230/300/385/500Hz
//gain:,0~24,Ӧ-12~+12dB
void WM8978_EQ2_Set(u8 cfreq,u8 gain)
{ 
	u16 regval=0;
	cfreq&=0X3;//޶Χ 
	if(gain>24)gain=24;
	gain=24-gain; 
	regval|=cfreq<<5;	//ýֹƵ 
	regval|=gain;		//	
 	WM8978_Write_Reg(19,regval);//R19,EQ2 	
}
//EQ3
//cfreq:Ƶ,0~3,ֱӦ:650/850/1100/1400Hz
//gain:,0~24,Ӧ-12~+12dB
void WM8978_EQ3_Set(u8 cfreq,u8 gain)
{ 
	u16 regval=0;
	cfreq&=0X3;//޶Χ 
	if(gain>24)gain=24;
	gain=24-gain; 
	regval|=cfreq<<5;	//ýֹƵ 
	regval|=gain;		//	
 	WM8978_Write_Reg(20,regval);//R20,EQ3 	
}
//EQ4
//cfreq:Ƶ,0~3,ֱӦ:1800/2400/3200/4100Hz
//gain:,0~24,Ӧ-12~+12dB
void WM8978_EQ4_Set(u8 cfreq,u8 gain)
{ 
	u16 regval=0;
	cfreq&=0X3;//޶Χ 
	if(gain>24)gain=24;
	gain=24-gain; 
	regval|=cfreq<<5;	//ýֹƵ 
	regval|=gain;		//	
 	WM8978_Write_Reg(21,regval);//R21,EQ4 	
}
//EQ5
//cfreq:Ƶ,0~3,ֱӦ:5300/6900/9000/11700Hz
//gain:,0~24,Ӧ-12~+12dB
void WM8978_EQ5_Set(u8 cfreq,u8 gain)
{ 
	u16 regval=0;
	cfreq&=0X3;//޶Χ 
	if(gain>24)gain=24;
	gain=24-gain; 
	regval|=cfreq<<5;	//ýֹƵ 
	regval|=gain;		//	
 	WM8978_Write_Reg(22,regval);//R22,EQ5 	
}

void WM8978_ALC_Set(u8 enable, u8 maxgain, u8 mingain)
{ 
	u16 regval;

	if(maxgain>7) maxgain=7;
	if(mingain>7) mingain=7;

	regval=WM8978_Read_Reg(32);
	if(enable)
		regval |= (3<<7);
	regval |= (maxgain<<3)|(mingain<<0);
 	WM8978_Write_Reg(32,regval);
}


void WM8978_Noise_Set(u8 enable,u8 gain)
{ 
	u16 regval;

	if(gain>7) gain=7;

	regval=WM8978_Read_Reg(35);
	regval = (enable<<3);
	regval|=gain;		//	
 	WM8978_Write_Reg(35,regval);//R18,EQ1 	
}

